/*
 * Copyright 2005, 2006 University of Leiden.
 *
 * This file is part of MIA+EWS.
 *
 * MIA+EWS is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * MIA+EWS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MIA; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/* $Id: simulate.h,v 1.4 2002/05/08 21:24:48 jdejong Exp $ */
/*******************************************************************************
* E.S.O. - VLT project
*
* "@(#) $Id: simulate.h,v 1.4 2002/05/08 21:24:48 jdejong Exp $"
*
* who       when      what
* --------  --------  ----------------------------------------------
* jdejong 2002-05-08 created
*/

/************************************************************************
 *
 *----------------------------------------------------------------------
 */

#ifndef _SIMULATE_H
#define _SIMULATE_H
/*----------------------------------------------------------------------- */
/* Routines and parameters for midi data simulation                       */
/*----------------------------------------------------------------------- */
#include <time.h>
#include <math.h>
#include <malloc.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "imaging_data.h"

/* Global Parameters for the simulation of midi data */
/* versions if main simulation program */
#ifdef SIMULATEMAIN
long   RON;        /* readout noise of detector */
double Sky;        /* sky (+telescope) brightness per msec */
double Coher;      /* Instrumental coherence factor (poln mismatch)*/
double Wavelength; /* center wavelength in microns */
double DW;         /* total range of wavelength in microns */
long   Nchan;      /* Number of frequency channels, assumed independent */
double Fract;      /* Fraction of the photons that go into each cross 
		      correlation */
double Counts;     /* counts per second per mJy */
double DetGain;    /* detector gain */
#else /* Not main program - declare external */
extern long   RON;        
extern double Sky;        
extern double Coher;      
extern double Wavelength; 
extern double DW;         
extern long   Nchan;      
extern double Fract;      
extern double Counts;
extern double DetGain;    /* detector gain */
#endif /* end of parameter definition */

/* Parameters for the simulation of midi data */
/*#define RON 2000         readout noise of detector */
/*#define SKY 6.0E7        sky (+telescope) brightness per msec */
/*#define COHER 0.60       Instrumental coherence factor (poln mismatch)*/
/*#define WAVELENGTH 10.0  center wavelength in microns */
/*#define DW   7.0         total range of wavelength in microns */
/*#define NCHAN 320        Number of frequency channels, assumed independent */
/*#define FRACT 0.35       Fraction of the photons that go into each cross correlation */
/*#define COUNTS 2.564e4   counts per second per mJy */

/* constants */
#define CLIGHT 2.997924562e8 /* speed of light */
#define PI    3.141592654    /* pi */
#define TWOPI 6.283185308    /* two pi */

imaging_data_table_def* 
make_data_table (fitsfile *fptr, int nx, int ny, int nreg, int maxstep, 
		 int *ierr);
/*----------------------------------------------------------------------- */
/*   Create FITS file and initialize table                                */
/*   Inputs:                                                              */
/*      fptr      cfitsio pointer to open FITS file                       */
/*      nx        Size of data array in x direction                       */
/*      ny        Size of data array in y direction                       */
/*      nreg      Number of regions                                       */
/*      maxstep   Maximum number of steps in obs. sequence.               */
/*   Output:                                                              */
/*      ierr      Error code: 0 => ok, checked on input                   */
/*   Returns:                                                             */
/*       imaging_data_table_def structure to use to write table           */
/*       NULL on failure                                                  */
/*----------------------------------------------------------------------- */

void simulate_data (imaging_data_row *row, float OPD_off,
		    float amplitude, int integ, float delay, 
		    float rate, float rmsOPD, float *gain, float *bg,
		    imaging_detector_table_def* detector);
/*----------------------------------------------------------------------- */
/*   Simulate telescope data                                              */
/*   Inputs:                                                              */
/*      row       row data structure to be filled in                      */
/*      OPD_off   OPD error for this sample                               */
/*      amplitude Amplitude of correlated signal in mJy                   */
/*      integ     integration time in msec                                */
/*      delay     Delay to add (microns)                                  */
/*      rate      Delay rate to add (microns/sec)                         */
/*      rmsOPD    RMS OPD noise per sample                                */
/*      gain      frequency gain factors for background, flux             */
/*      bg        background factor per frequency channel                 */
/*      detector  imaging_detector table definition structure             */
/*----------------------------------------------------------------------- */

float GaussNoise (float sigma);
/*----------------------------------------------------------------------- */
/*  Generate zero mean Gaussian noise (not very sophisticated)            */
/*  Must be initialized before first use                                  */
/*   Input:                                                               */
/*      sigma  Target sigma for the noise, if < 0.0 initialize only       */
/*   Returns:                                                             */
/*      a randum number                                                   */
/*----------------------------------------------------------------------- */

FILE* OPDfile_open(char *OPDfile, double tau0);
/*----------------------------------------------------------------------- */
/*  Open an OPD model file                                                */
/*  An OPD model file contains a series of OPD delays.                    */
/*  Comment lines start with "!".                                         */
/*  Each (noncomment) line contains a time in sec and a delay in microns  */
/*  as floats.                                                            */
/*     If the first non comment line starts with "TAU_0 =", then the      */
/*  following value is assumed to the the Tau_0 of the file.              */
/*  In this case the output OPD values will be scaled to the input        */
/*  value of tau0.                                                        */
/*   Input:                                                               */
/*      OPDfile    Name of OPDfile                                        */
/*      tau0       Desired tau_0 of output, <=0 -> ignore                 */
/*   Returns:                                                             */
/*      File I/O pointer                                                  */
/*----------------------------------------------------------------------- */

float OPDfile_read(FILE* hFile, double _time);
/*----------------------------------------------------------------------- */
/*  Read/interpolate the next delay from the sequence.                    */
/*  OPDfile_open must have been called first                              */
/*   Input:                                                               */
/*      hFile   File I/O pointer from OPDfile_open                        */
/*      time    Sequence time in seconds for the OPD value                */
/*   Returns:                                                             */
/*      The next OPD in the sequence (microns), 0.0 if something is wrong */
/*----------------------------------------------------------------------- */

void  OPDfile_close(FILE* hFile);
/*----------------------------------------------------------------------- */
/*  Close an OPDfile Opened with OPDfile_open                             */
/*   Input:                                                               */
/*      hFile   File I/O pointer from OPDfile_open                        */
/*----------------------------------------------------------------------- */

#endif /* end SIMULATE_H */





/* */
